/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { OrganisationsEinheitContainerComponent } from '@admin-client/organisations-einheit';
import { AdminOrganisationsEinheit, AdminOrganisationsEinheitService } from '@admin-client/organisations-einheit-shared';
import { createStateResource, StateResource, ToEmbeddedResourcesPipe } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getMockComponent, Mock, mock } from '@alfa-client/test-utils';
import { OzgcloudDialogService, UiModule } from '@alfa-client/ui';
import { OrganisationsEinheitResource } from '@alfa-client/zustaendige-stelle-shared';
import { CommonModule } from '@angular/common';
import { ViewContainerRef } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createAdminOrganisationsEinheit } from 'libs/admin/organisations-einheit-shared/src/test/organisations-einheit';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createOrganisationsEinheitResource } from '../../../../../zustaendige-stelle-shared/test/organisations-einheit';
import { OrganisationsEinheitListComponent } from './organisations-einheit-list/organisations-einheit-list.component';

describe('OrganisationsEinheitContainerComponent', () => {
  let component: OrganisationsEinheitContainerComponent;
  let fixture: ComponentFixture<OrganisationsEinheitContainerComponent>;

  let organisationsEinheitService: Mock<AdminOrganisationsEinheitService>;
  let dialogService: Mock<OzgcloudDialogService>;
  let vieContainerRef: Mock<ViewContainerRef>;

  const organisationsEinheit: AdminOrganisationsEinheit = createAdminOrganisationsEinheit();
  const organisationsEinheitListStateResource: StateResource<AdminOrganisationsEinheit[]> = createStateResource([
    organisationsEinheit,
  ]);

  const listSelector: string = getDataTestIdOf('organisations-einheit-list');

  beforeEach(() => {
    organisationsEinheitService = {
      ...mock(AdminOrganisationsEinheitService),
      getAll: jest.fn().mockReturnValue(of(organisationsEinheitListStateResource)),
      create: jest.fn().mockReturnValue(of(false)),
      refresh: jest.fn(),
    };
    dialogService = mock(OzgcloudDialogService);
    vieContainerRef = mock(ViewContainerRef as any);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ToEmbeddedResourcesPipe],
      imports: [CommonModule, UiModule, OrganisationsEinheitContainerComponent, MockComponent(OrganisationsEinheitListComponent)],
      providers: [
        { provide: AdminOrganisationsEinheitService, useValue: organisationsEinheitService },
        { provide: OzgcloudDialogService, useValue: dialogService },
        { provide: ViewContainerRef, useValue: vieContainerRef },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationsEinheitContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngOnInit', () => {
      it('should call organisationsEinheitService getAll', (done) => {
        component.ngOnInit();

        component.organisationsEinheitenStateResource$.subscribe(() => {
          expect(organisationsEinheitService.getAll).toHaveBeenCalled();
          done();
        });
      });

      describe('ngOnDestroy', () => {
        it('should refresh organisationseinheiten list state resource', () => {
          component.ngOnDestroy();

          expect(organisationsEinheitService.refresh).toHaveBeenCalled();
        });
      });
    });

    describe('openDialog', () => {
      beforeEach(() => {
        component.handleDialogResponse = jest.fn();
      });

      it('should open dialog', () => {
        dialogService.openFullScreenDialog = jest.fn().mockReturnValue({ closed: of(organisationsEinheit) });

        component.openDialog();

        expect(dialogService.openFullScreenDialog).toHaveBeenCalled();
      });

      it('should handle dialog response', () => {
        dialogService.openFullScreenDialog = jest.fn().mockReturnValue({ closed: of(organisationsEinheit) });

        component.openDialog();

        expect(component.handleDialogResponse).toHaveBeenCalled();
      });

      it('should not handle response if its undefined', () => {
        dialogService.openFullScreenDialog = jest.fn().mockReturnValue({ closed: of(undefined) });

        component.openDialog();

        expect(component.handleDialogResponse).not.toHaveBeenCalled();
      });
    });

    describe('handleDialogResponse', () => {
      const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

      beforeEach(() => {
        dialogService.openFullScreenDialog = jest.fn().mockReturnValue({ closed: of(organisationsEinheit) });
      });

      it('should call organisationsEinheitService create', () => {
        component.handleDialogResponse(organisationsEinheitResource);

        expect(organisationsEinheitService.create).toHaveBeenCalledWith({ name: organisationsEinheitResource.name });
      });

      it('should set createOrganisationsEinheitProgress$', () => {
        component.handleDialogResponse(organisationsEinheitResource);

        expect(component.createOrganisationsEinheitProgress$).toBeTruthy();
      });
    });
  });

  describe('template', () => {
    describe('organisationsEinheiten list', () => {
      it('should show list', () => {
        existsAsHtmlElement(fixture, listSelector);
      });

      it('should have organisationsEinheitResources', () => {
        const listComponent: OrganisationsEinheitListComponent = getMockComponent(fixture, OrganisationsEinheitListComponent);
        expect(listComponent.organisationsEinheitList).toBeTruthy();
      });
    });
  });
});
