/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootLinkRel } from '@alfa-client/api-root-shared';
import { GetUrlPipe, HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import { getElementFromDomRoot } from '@alfa-client/test-utils';
import { UiModule } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import {
  DropdownMenuComponent,
  DropdownMenuLinkItemComponent,
  DropdownMenuTextItemComponent,
  FileIconComponent,
  HelpIconComponent,
} from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { createApiRootResource } from '../../../../../libs/api-root-shared/test/api-root';
import { DocumentationComponent } from './documentation/documentation.component';
import { HelpButtonComponent } from './help-button/help-button.component';
import { HelpMenuComponent } from './help-menu.component';

describe('HelpMenuComponent', () => {
  let component: HelpMenuComponent;
  let fixture: ComponentFixture<HelpMenuComponent>;

  const helpMenuButton: string = getDataTestIdOf('help-menu-button');
  const impressumLink: string = getDataTestIdOf('impressum');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        HelpMenuComponent,
        HasLinkPipe,
        GetUrlPipe,
        MockComponent(DocumentationComponent),
        MockComponent(FileIconComponent),
        MockComponent(HelpIconComponent),
        MockComponent(DropdownMenuTextItemComponent),
        MockComponent(DropdownMenuComponent),
        MockComponent(DropdownMenuLinkItemComponent),
        MockComponent(HelpButtonComponent),
      ],
      imports: [NoopAnimationsModule, UiModule],
    }).compileComponents();

    fixture = TestBed.createComponent(HelpMenuComponent);
    component = fixture.componentInstance;
    component.apiRootStateResource = createStateResource(createApiRootResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('input', () => {
    describe('apiRootStateResource', () => {
      it('should set buttonEnabled to true', () => {
        component.hasMenuItems = jest.fn().mockReturnValue(true);
        component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.DOCUMENTATIONS]));

        fixture.detectChanges();

        expect(component.enabled).toBeTruthy();
      });

      it('should set buttonEnabled to false', () => {
        component.hasMenuItems = jest.fn().mockReturnValue(false);
        component.apiRootStateResource = createStateResource(createApiRootResource());

        fixture.detectChanges();

        expect(component.enabled).toBeFalsy();
      });
    });
  });

  describe('component', () => {
    describe('hasMenuItems', () => {
      it('should return true if documentation Link exists', () => {
        component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.DOCUMENTATIONS]));

        const res: boolean = component.hasMenuItems();

        expect(res).toBeTruthy();
      });

      it('should return true if impressumUrl exists', () => {
        component.apiRootStateResource = createStateResource(createApiRootResource());

        const res: boolean = component.hasMenuItems();

        expect(res).toBeTruthy();
      });

      it('should return false', () => {
        component.apiRootStateResource = createStateResource(createApiRootResource());
        component.apiRootStateResource.resource.impressumUrl = null;

        const res: boolean = component.hasMenuItems();

        expect(res).toBeFalsy();
      });
    });
  });

  describe('template', () => {
    describe('help menu button', () => {
      it('should be hidden', () => {
        component.enabled = false;

        fixture.detectChanges();

        const item = getElementFromDomRoot(fixture, helpMenuButton);
        expect(item).not.toBeInstanceOf(HTMLElement);
      });

      it('should be shown', () => {
        component.enabled = true;

        fixture.detectChanges();

        const item = getElementFromDomRoot(fixture, helpMenuButton);
        expect(item).toBeInstanceOf(HTMLElement);
      });
    });

    describe('alfa-documentation', () => {
      it('should be hidden if no documentation link', () => {
        const documentation = getElementFromDomRoot(fixture, getDataTestIdOf('documentation'));

        expect(documentation).not.toBeInstanceOf(HTMLElement);
      });

      it('should be shown', () => {
        component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.DOCUMENTATIONS]));

        fixture.detectChanges();

        const documentation = getElementFromDomRoot(fixture, getDataTestIdOf('documentation'));
        expect(documentation).toBeInstanceOf(HTMLElement);
      });
    });

    describe('ods-dropdown-menu-link', () => {
      it('should be hidden if no impressumUrl', () => {
        component.apiRootStateResource.resource.impressumUrl = null;

        fixture.detectChanges();

        const impressum = getElementFromDomRoot(fixture, impressumLink);
        expect(impressum).not.toBeInstanceOf(HTMLElement);
      });

      it('should be shown if impressumUrl', () => {
        const impressum = getElementFromDomRoot(fixture, impressumLink);
        expect(impressum).toBeInstanceOf(HTMLElement);
      });
    });
  });
});
