/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AktenzeichenEditDialogComponent } from './aktenzeichen-edit-dialog.component';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { mock } from '@alfa-client/test-utils';
import {
  OzgcloudPasteTextButtonComponent,
  OzgcloudStrokedButtonWithSpinnerComponent,
  SnackBarService,
  TextEditorComponent,
} from '@alfa-client/ui';
import { MockComponent, MockDirective } from 'ng-mocks';
import { VorgangService } from '@alfa-client/vorgang-shared';
import {
  MAT_DIALOG_DATA,
  MatDialogActions,
  MatDialogContent,
  MatDialogRef,
} from '@angular/material/dialog';
import { MatIcon } from '@angular/material/icon';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import {
  createEmptyStateResource,
  hasContent,
  isClipboardReadSupported,
} from '@alfa-client/tech-shared';
import { EMPTY, NEVER } from 'rxjs';
import { FormBuilder, ReactiveFormsModule, UntypedFormControl } from '@angular/forms';
import { cold, hot } from 'jest-marbles';
import {
  createCommandErrorResource,
  createCommandResource,
} from '../../../../command-shared/test/command';
import {
  COMMAND_ERROR_MESSAGES,
  CommandErrorMessage,
  hasCommandError,
  isSuccessfulDone,
} from '@alfa-client/command-shared';
import { AktenzeichenEditDialogMessages } from './aktenzeichen-edit-dialog.message';
import { AktenzeichenEditDialogFormservice } from './aktenzeichen-edit-dialog.formservice';

jest.mock('@alfa-client/tech-shared');
const isClipboardReadSupportedMock = isClipboardReadSupported as jest.Mock;
jest.mock('@alfa-client/command-shared');
const hasErrorMock = hasCommandError as jest.Mock;
const isSuccessfulDoneMock = isSuccessfulDone as jest.Mock;
const hasContentMock = hasContent as jest.Mock;
const createEmptyStateResourceMock = createEmptyStateResource as jest.Mock;

describe('AktenzeichenEditDialogComponent', () => {
  let component: AktenzeichenEditDialogComponent;
  let fixture: ComponentFixture<AktenzeichenEditDialogComponent>;

  const formService = mock(AktenzeichenEditDialogFormservice);
  formService.valueChanges.mockReturnValue(NEVER);

  const snackBarService = mock(SnackBarService);
  const vorgangService = mock(VorgangService);
  const matDialogRef = mock(MatDialogRef);

  const vorgang = createVorgangWithEingangResource();

  beforeEach(async () => {
    TestBed.overrideComponent(AktenzeichenEditDialogComponent, {
      set: {
        providers: [
          {
            provide: AktenzeichenEditDialogFormservice,
            useValue: formService,
          },
        ],
      },
    });
    await TestBed.configureTestingModule({
      declarations: [
        AktenzeichenEditDialogComponent,
        MockDirective(MatDialogContent),
        MockDirective(MatDialogActions),
        MockComponent(MatIcon),
        MockComponent(TextEditorComponent),
        MockComponent(OzgcloudPasteTextButtonComponent),
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
      ],
      imports: [ReactiveFormsModule],
      providers: [
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
        {
          provide: SnackBarService,
          useValue: snackBarService,
        },
        {
          provide: MAT_DIALOG_DATA,
          useValue: { data: { vorgang } },
        },
        {
          provide: MatDialogRef,
          useValue: matDialogRef,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(AktenzeichenEditDialogComponent);
    component = fixture.componentInstance;
    component.vorgang = vorgang;
    jest.spyOn(component, 'form', 'get').mockReturnValue(
      new FormBuilder().group({
        [AktenzeichenEditDialogFormservice.AKTENZEICHEN_FIELD]: new UntypedFormControl(null),
      }),
    );
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should check if paste is supported', () => {
      component.ngOnInit();

      expect(isClipboardReadSupportedMock).toHaveBeenCalled();
    });

    it('should set aktenzeichen from Vorgang on form', () => {
      component.ngOnInit();

      expect(formService.setAktenzeichen).toHaveBeenCalledWith(vorgang.aktenzeichen);
    });

    describe('hasAktenzeichen$', () => {
      it('should emit true', () => {
        hasContentMock.mockReturnValue(true);

        component.ngOnInit();

        expect(component.hasAktenzeichen$).toBeObservable(hot('a', { a: true }));
      });

      it('should emit false', () => {
        hasContentMock.mockReturnValue(false);

        component.ngOnInit();

        expect(component.hasAktenzeichen$).toBeObservable(hot('a', { a: false }));
      });
    });
  });

  describe('onCancel', () => {
    it('should close dialog', () => {
      component.onCancel();

      expect(matDialogRef.close).toHaveBeenCalled();
    });
  });

  describe('onSubmit', () => {
    formService.submit = jest.fn();

    it('should set vorgang on form service', () => {
      formService.submit.mockReturnValue(EMPTY);

      component.onSubmit();

      expect(formService.setVorgangWithEingangResource).toHaveBeenCalledWith(vorgang);
    });

    describe('setAktenzeichenPending$', () => {
      beforeEach(() => {
        formService.submit = jest.fn();
        formService.submit.mockReturnValue(NEVER);
        component.onResponse = jest.fn();
        createEmptyStateResourceMock.mockReturnValue({ resource: null });
      });

      it('should call onResponse', (done) => {
        component.onSubmit();

        component.setAktenzeichenPending$.subscribe(() => {
          expect(component.onResponse).toHaveBeenCalledWith(null);
          done();
        });
      });

      it('should emit state resource', () => {
        component.onSubmit();

        expect(component.setAktenzeichenPending$).toBeObservable(cold('a', { a: { resource: null } }));
      });
    });
  });

  describe('onResponse', () => {
    it('should show snack bar on error command', () => {
      const commandResource = createCommandErrorResource();
      hasErrorMock.mockReturnValue(true);
      component.showSnackbar = jest.fn();

      component.onResponse(commandResource);

      expect(component.showSnackbar).toHaveBeenCalledWith(commandResource);
    });

    it('should not show snack bar', () => {
      const commandResource = createCommandResource();
      hasErrorMock.mockReturnValue(false);
      isSuccessfulDoneMock.mockReturnValue(false);
      component.showSnackbar = jest.fn();

      component.onResponse(commandResource);

      expect(component.showSnackbar).not.toHaveBeenCalledWith(commandResource);
    });

    it('should reload current vorgang', () => {
      const commandResource = createCommandResource();
      hasErrorMock.mockReturnValue(false);
      isSuccessfulDoneMock.mockReturnValue(true);

      component.onResponse(commandResource);

      expect(vorgangService.reloadCurrentVorgang).toHaveBeenCalled();
    });

    it('should close dialog', () => {
      const commandResource = createCommandResource();
      hasErrorMock.mockReturnValue(false);
      isSuccessfulDoneMock.mockReturnValue(true);

      component.onResponse(commandResource);

      expect(matDialogRef.close).toHaveBeenCalled();
    });
  });

  describe('showSnackbar', () => {
    beforeEach(() => {
      snackBarService.showError.mockClear();
    });

    it('should show error', () => {
      hasErrorMock.mockReturnValue(true);
      jest.spyOn(component, 'getErrorMessage').mockReturnValue('error');

      component.showSnackbar(createCommandErrorResource());

      expect(snackBarService.showError).toHaveBeenCalledWith('error');
    });

    it('should not show error', () => {
      hasErrorMock.mockReturnValue(false);

      component.showSnackbar(createCommandErrorResource());

      expect(snackBarService.showError).not.toHaveBeenCalled();
    });

    it('should getErrorMessage', () => {
      const commandResource = createCommandErrorResource();
      hasErrorMock.mockReturnValue(true);
      jest.spyOn(component, 'getErrorMessage').mockReturnValue('error');

      component.showSnackbar(commandResource);

      expect(component.getErrorMessage).toHaveBeenCalledWith(commandResource);
    });
  });

  describe('getErrorMessage', () => {
    it('should return error message for SET_AKTENZEICHEN', () => {
      const msg = component.getErrorMessage(createCommandResource());

      expect(msg).toEqual(AktenzeichenEditDialogMessages.SET_FAILED);
    });

    it('should return error message for CONCURRENT_MODIFICATION', () => {
      const msg = component.getErrorMessage(createCommandErrorResource());

      expect(msg).toEqual(COMMAND_ERROR_MESSAGES[CommandErrorMessage.CONCURRENT_MODIFICATION]);
    });
  });

  describe('paste', () => {
    it('should set aktenzeichen on form', () => {
      component.paste('abc');

      expect(formService.setAktenzeichen).toHaveBeenCalledWith('abc');
    });
  });
});
