/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { createEmptyStateResource, createStateResource, HasLinkPipe, StateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { VorgangCommandService, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { CheckIconComponent } from '@ods/system';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestIdAttributeOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { Observable, of } from 'rxjs';
import { AbschliessenButtonComponent } from './abschliessen-button.component';

describe('AbschliessenButtonComponent', () => {
  let component: AbschliessenButtonComponent;
  let fixture: ComponentFixture<AbschliessenButtonComponent>;

  const abschliessenButton: string = getDataTestIdAttributeOf('abschliessen-button');
  const abschliessenIconButton: string = getDataTestIdAttributeOf('abschliessen-icon-button');

  const vorgangCommandService = {
    ...mock(VorgangCommandService),
    getCommandStateResources: () => of(createStateResource(createCommandResource())),
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        AbschliessenButtonComponent,
        MockComponent(ButtonWithSpinnerComponent),
        MockComponent(CheckIconComponent),
        HasLinkPipe,
      ],
      providers: [
        {
          provide: VorgangCommandService,
          useValue: vorgangCommandService,
        },
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(AbschliessenButtonComponent);
    component = fixture.componentInstance;
    component.vorgang = createVorgangWithEingangResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service', () => {
      component.ngOnInit();

      expect(vorgangCommandService.getAbschliessenCommand).toHaveBeenCalled();
    });
  });

  describe('abschliessen', () => {
    const command: CommandResource = createCommandResource();
    const comandStateResource$: Observable<StateResource<CommandResource>> = of(createStateResource(command));

    beforeEach(() => {
      vorgangCommandService.abschliessen.mockReturnValue(comandStateResource$);
    });

    it('should call vorgang service', () => {
      component.abschliessen();

      expect(vorgangCommandService.abschliessen).toHaveBeenCalled();
    });

    it('should assign response', () => {
      component.commandStateResource$ = of(createEmptyStateResource<CommandResource>());

      component.abschliessen();

      expect(component.commandStateResource$).toBe(comandStateResource$);
    });
  });

  describe('abschliessen button', () => {
    beforeEach(() => {
      component.showAsIconButton = false;
      fixture.detectChanges();
    });

    it('should be hidden', () => {
      component.isVisible = false;

      fixture.detectChanges();
      const buttonElement = fixture.nativeElement.querySelector(abschliessenButton);

      expect(buttonElement).not.toBeInstanceOf(HTMLElement);
    });

    it('should be visible', () => {
      component.isVisible = true;

      fixture.detectChanges();
      const buttonElement = fixture.nativeElement.querySelector(abschliessenButton);

      expect(buttonElement).toBeInstanceOf(HTMLElement);
    });
  });

  describe('abschliessen icon button', () => {
    beforeEach(() => {
      component.showAsIconButton = true;
      fixture.detectChanges();
    });

    it('should be hidden', () => {
      component.isVisible = false;

      fixture.detectChanges();
      const buttonElement = fixture.nativeElement.querySelector(abschliessenIconButton);

      expect(buttonElement).not.toBeInstanceOf(HTMLElement);
    });

    it('should be visible', () => {
      component.isVisible = true;

      fixture.detectChanges();
      const buttonElement = fixture.nativeElement.querySelector(abschliessenIconButton);

      expect(buttonElement).toBeInstanceOf(HTMLElement);
    });
  });

  describe('set vorgang', () => {
    it('should set vorgang with eingang resource', () => {
      const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

      component.vorgang = vorgang;

      expect(component.vorgangWithEingangResource).toBe(vorgang);
    });

    it('should set visible to true', () => {
      const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ABSCHLIESSEN]);
      component.vorgang = vorgang;

      const isVisible: boolean = component.isVisible;

      expect(isVisible).toBeTruthy();
    });

    it('should set visible to false', () => {
      const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
      component.vorgang = vorgang;

      const isVisible: boolean = component.isVisible;

      expect(isVisible).toBeFalsy();
    });

    it('should set visible to false if have createBescheidDraft link', () => {
      const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.ABSCHLIESSEN,
        VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
      ]);
      component.vorgang = vorgang;

      const isVisible: boolean = component.isVisible;

      expect(isVisible).toBeFalsy();
    });

    it('should set visible to false if have bescheidDraft link', () => {
      const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.ABSCHLIESSEN,
        VorgangWithEingangLinkRel.BESCHEID_DRAFT,
      ]);
      component.vorgang = vorgang;

      const isVisible: boolean = component.isVisible;

      expect(isVisible).toBeFalsy();
    });
  });
});
