/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { createStateResource, HasLinkPipe, StateResource } from '@alfa-client/tech-shared';
import { getElementComponentFromFixtureByCss, mock, notExistsAsHtmlElement, triggerEvent } from '@alfa-client/test-utils';
import { VorgangCommandService, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { CheckCircleIconComponent, TooltipDirective } from '@ods/system';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestIdAttributeOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { AnnehmenButtonComponent } from './annehmen-button.component';

describe('AnnehmenButtonComponent', () => {
  let component: AnnehmenButtonComponent;
  let fixture: ComponentFixture<AnnehmenButtonComponent>;

  const annehmenButton: string = getDataTestIdAttributeOf('annehmen-button');
  const annehmenIconButton: string = getDataTestIdAttributeOf('annehmen-icon-button');

  const vorgangCommandService = {
    ...mock(VorgangCommandService),
    getCommandStateResources: () => of(createStateResource(createCommandResource())),
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        AnnehmenButtonComponent,
        HasLinkPipe,
        TooltipDirective,
        MockComponent(ButtonWithSpinnerComponent),
        MockComponent(CheckCircleIconComponent),
      ],
      providers: [
        {
          provide: VorgangCommandService,
          useValue: vorgangCommandService,
        },
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(AnnehmenButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngOnInit', () => {
      it('should call service', () => {
        component.ngOnInit();

        expect(vorgangCommandService.getAnnehmenCommand).toHaveBeenCalled();
      });
    });

    describe('annehmen', () => {
      it('should call vorgang service', () => {
        component.annehmen();

        expect(vorgangCommandService.annehmen).toHaveBeenCalled();
      });
    });
  });

  describe('template', () => {
    const state: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ANNEHMEN]);
      component.commandStateResource$ = of(state);
      fixture.detectChanges();
    });

    describe('annehmen button', () => {
      beforeEach(() => {
        component.showAsIconButton = false;
        fixture.detectChanges();
      });

      it('should not exist', () => {
        component.vorgang = createVorgangWithEingangResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, annehmenButton);
      });

      it('should exist with input', () => {
        fixture.detectChanges();

        const button: ButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(fixture, annehmenButton);

        expect(button.stateResource).toBe(state);
      });

      it('should call annehmen on click', () => {
        component.annehmen = jest.fn();

        triggerEvent({ fixture, name: 'clickEmitter', elementSelector: annehmenButton });

        expect(component.annehmen).toHaveBeenCalled();
      });
    });

    describe('annehmen icon button', () => {
      beforeEach(() => {
        component.showAsIconButton = true;
        fixture.detectChanges();
      });

      it('should not exist', () => {
        component.vorgang = createVorgangWithEingangResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, annehmenIconButton);
      });

      it('should exist with input', () => {
        fixture.detectChanges();

        const button: ButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(fixture, annehmenIconButton);

        expect(button.stateResource).toBe(state);
      });

      it('should call annehmen on click', () => {
        component.annehmen = jest.fn();

        triggerEvent({ fixture, name: 'clickEmitter', elementSelector: annehmenIconButton });

        expect(component.annehmen).toHaveBeenCalled();
      });
    });
  });
});
