/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidResource, BescheidService, BescheidWizardDialogResult } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { createEmptyStateResource, createStateResource, HasLinkPipe, StateResource } from '@alfa-client/tech-shared';
import {
  getElementComponentFromFixtureByCss,
  getMockComponent,
  Mock,
  mock,
  notExistsAsHtmlElement,
  tooltipExistsWithText,
  triggerEvent,
} from '@alfa-client/test-utils';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { BescheidenDialogData } from '@alfa-client/vorgang-detail';
import {
  VorgangCommandService,
  VorgangService,
  VorgangWithEingangLinkRel,
  VorgangWithEingangResource,
} from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { StampIconComponent, TooltipDirective } from '@ods/system';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { Observable, of } from 'rxjs';
import { createBescheidResource } from '../../../../../bescheid-shared/src/test/bescheid';
import { VorgangDetailBescheidenComponent } from '../../vorgang-detail-page/vorgang-detail-bescheiden/vorgang-detail-bescheiden.component';
import { BescheidenButtonComponent } from './bescheiden-button.component';

describe('BescheidenButtonComponent', () => {
  let component: BescheidenButtonComponent;
  let fixture: ComponentFixture<BescheidenButtonComponent>;

  const bescheidenButton: string = getDataTestIdOf('bescheiden-button');
  const bescheidenIconButton: string = getDataTestIdOf('bescheiden-icon-button');

  let vorgangCommandService: Mock<VorgangCommandService>;
  let ozgcloudDialogService: Mock<OzgcloudDialogService>;
  let bescheidService: Mock<BescheidService>;
  let vorgangService: Mock<VorgangService>;

  const dialogRef = <DialogRef<VorgangDetailBescheidenComponent>>{};

  beforeEach(() => {
    vorgangCommandService = mock(VorgangCommandService);
    ozgcloudDialogService = mock(OzgcloudDialogService);
    bescheidService = mock(BescheidService);
    vorgangService = mock(VorgangService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidenButtonComponent,
        HasLinkPipe,
        TooltipDirective,
        MockComponent(ButtonWithSpinnerComponent),
        MockComponent(StampIconComponent),
      ],
      providers: [
        {
          provide: VorgangCommandService,
          useValue: vorgangCommandService,
        },
        {
          provide: OzgcloudDialogService,
          useValue: ozgcloudDialogService,
        },
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidenButtonComponent);
    component = fixture.componentInstance;
    ozgcloudDialogService.openWizard = jest.fn().mockReturnValue(dialogRef);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngOnInit', () => {
      it('should call service', () => {
        component.ngOnInit();

        expect(vorgangCommandService.getBeschiedenCommand).toHaveBeenCalled();
      });
    });

    describe('bescheiden', () => {
      describe('should open bescheid wizard', () => {
        beforeEach(() => {
          component.openBescheidenWizard = jest.fn();
          component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT]);
        });

        it('should open bescheid wizard when create bescheid draft link exists', () => {
          component.bescheiden();

          expect(component.openBescheidenWizard).toHaveBeenCalled();
        });

        it('should open bescheid wizard when bescheid draft exists', () => {
          component.bescheiden();

          expect(component.openBescheidenWizard).toHaveBeenCalled();
        });
      });

      describe('should do bescheiden', () => {
        const command: CommandResource = createCommandResource();
        const comandStateResource$: Observable<StateResource<CommandResource>> = of(createStateResource(command));

        beforeEach(() => {
          vorgangCommandService.bescheiden.mockReturnValue(comandStateResource$);
        });

        it('should call vorgangCommandService.bescheiden', () => {
          component.bescheiden();

          expect(vorgangCommandService.bescheiden).toHaveBeenCalled();
        });

        it('should assign response', () => {
          component.commandStateResource$ = of(createEmptyStateResource<CommandResource>());

          component.bescheiden();

          expect(component.commandStateResource$).toBe(comandStateResource$);
        });
      });
    });

    // TODO: Use this version after completed Bescheid refactoring and delete the other version below.
    // describe('openBescheidenWizard', () => {
    //   let dialogRefMock: DialogRefMock<BescheidWizardDialogResult>;
    //
    //   beforeEach(() => {
    //     dialogRefMock = createDialogRefMock<BescheidWizardDialogResult>();
    //     ozgcloudDialogService.openWizard.mockReturnValue(dialogRefMock);
    //   });
    //
    //   it('should open wizard dialog', () => {
    //     const vorgang = createVorgangWithEingangResource();
    //     component.vorgang = vorgang;
    //
    //     component.openBescheidenWizard();
    //
    //     expect(ozgcloudDialogService.openWizard).toHaveBeenCalledWith(BescheidWizardContainerComponent, {
    //       vorgangWithEingangResource: vorgang,
    //     });
    //   });
    //
    //   it('should handleBescheidWizardClosed', () => {
    //     component.handleBescheidWizardClosed = jest.fn();
    //     const dialogResult: BescheidWizardDialogResult = { reloadVorgang: true };
    //     dialogRefMock.closed = of(dialogResult);
    //
    //     component.openBescheidenWizard();
    //
    //     expect(component.handleBescheidWizardClosed).toHaveBeenCalledWith(dialogResult);
    //   });
    // });

    describe('openBescheidenWizard', () => {
      it('should init', () => {
        component.openBescheidenWizard();

        expect(bescheidService.init).toHaveBeenCalled();
      });

      it('should open bescheiden dialog with existing draft', () => {
        component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
        component.openBescheidenDialogWithExistingDraft = jest.fn();

        component.openBescheidenWizard();

        expect(component.openBescheidenDialogWithExistingDraft).toHaveBeenCalled();
      });

      it('should open bescheiden dialog with new draft', () => {
        component.vorgang = createVorgangWithEingangResource();
        component.openBescheidDialogWithNewDraft = jest.fn();

        component.openBescheidenWizard();

        expect(component.openBescheidDialogWithNewDraft).toHaveBeenCalled();
      });
    });

    describe('openBescheidDialogWithNewDraft', () => {
      it('should open wizard', () => {
        component.vorgang = createVorgangWithEingangResource();
        component.openDialog = jest.fn();

        component.openBescheidDialogWithNewDraft();

        expect(component.openDialog).toHaveBeenCalledWith({
          vorgangWithEingangResource: component.vorgang,
          bescheidDraftResource: null,
        });
      });
    });

    describe('openBescheidenDialogWithExistingDraft', () => {
      const bescheidDraftResource: BescheidResource = createBescheidResource();
      const bescheidDraftStateResource: StateResource<BescheidResource> = createStateResource(bescheidDraftResource);
      const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.BESCHEID_DRAFT,
      ]);

      beforeEach(() => {
        component.vorgang = vorgangWithEingangResource;
        bescheidService.getBescheidDraftIfExists.mockReturnValue(of(bescheidDraftStateResource));
      });

      it('should open wizard if bescheid draft loaded', () => {
        component.openBescheidenDialogWithExistingDraft();

        expect(ozgcloudDialogService.openWizard).toHaveBeenCalledWith(VorgangDetailBescheidenComponent, {
          bescheidDraftResource,
          vorgangWithEingangResource,
        });
      });

      it('should not open wizard if bescheid draft not loaded', () => {
        bescheidService.getBescheidDraftIfExists.mockReturnValue(of(createEmptyStateResource()));

        component.openBescheidenDialogWithExistingDraft();

        expect(ozgcloudDialogService.openWizard).not.toHaveBeenCalled();
      });

      it('should not open wizard on loading bescheid draft', () => {
        bescheidService.getBescheidDraftIfExists.mockReturnValue(
          of({ ...createStateResource(createBescheidResource()), loading: true }),
        );

        component.openBescheidenDialogWithExistingDraft();

        expect(ozgcloudDialogService.openWizard).not.toHaveBeenCalled();
      });
    });

    describe('openDialog', () => {
      const bescheidDraftResource: BescheidResource = createBescheidResource();
      const bescheidDraftStateResource: StateResource<BescheidResource> = createStateResource(bescheidDraftResource);
      const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.BESCHEID_DRAFT,
      ]);

      const dialogData: BescheidenDialogData = {
        bescheidDraftResource: bescheidDraftResource,
        vorgangWithEingangResource: vorgangWithEingangResource,
      };

      beforeEach(() => {
        component.vorgang = vorgangWithEingangResource;
        bescheidService.getBescheidDraftIfExists.mockReturnValue(of(bescheidDraftStateResource));
      });

      it('should call ozgcloudDialogService.openWizard', () => {
        component.openDialog(dialogData);

        expect(ozgcloudDialogService.openWizard).toHaveBeenCalledWith(VorgangDetailBescheidenComponent, dialogData);
      });
    });

    describe('handleBescheidWizardClosed', () => {
      it('should reload current vorgang', () => {
        component.handleBescheidWizardClosed({ reloadVorgang: true });

        expect(vorgangService.reloadCurrentVorgang).toHaveBeenCalled();
      });

      it('should not reload current vorgang', () => {
        component.handleBescheidWizardClosed({ reloadVorgang: false });

        expect(vorgangService.reloadCurrentVorgang).not.toHaveBeenCalled();
      });

      it.each([null, undefined])(
        'should not reload current vorgang if result nil',
        (dialogResult: BescheidWizardDialogResult) => {
          component.handleBescheidWizardClosed(dialogResult);

          expect(vorgangService.reloadCurrentVorgang).not.toHaveBeenCalled();
        },
      );
    });
  });

  describe('template', () => {
    const state: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEIDEN]);
      component.commandStateResource$ = of(state);

      fixture.detectChanges();
    });

    describe('bescheiden button', () => {
      beforeEach(() => {
        component.showAsIconButton = false;
        fixture.detectChanges();
      });

      it('should not exits', () => {
        component.vorgang = createVorgangWithEingangResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidenButton);
      });

      it('should exist with input', () => {
        const button: ButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(fixture, bescheidenButton);

        expect(button.stateResource).toBe(state);
      });

      it('should get "Bescheid" text', () => {
        const button: ButtonWithSpinnerComponent = getMockComponent(fixture, ButtonWithSpinnerComponent);
        expect(button.text).toBe('Bescheiden');
      });

      it('should get "Bescheiden fortsetzen" text', () => {
        component.vorgang = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.BESCHEIDEN,
          VorgangWithEingangLinkRel.BESCHEID_DRAFT,
        ]);

        fixture.detectChanges();

        const button: ButtonWithSpinnerComponent = getMockComponent(fixture, ButtonWithSpinnerComponent);
        expect(button.text).toBe('Bescheiden fortsetzen');
      });

      it('should call bescheiden on click', () => {
        component.bescheiden = jest.fn();

        triggerEvent({ fixture, name: 'clickEmitter', elementSelector: bescheidenButton });

        expect(component.bescheiden).toHaveBeenCalled();
      });
    });

    describe('bescheiden icon button', () => {
      beforeEach(() => {
        component.showAsIconButton = true;
        fixture.detectChanges();
      });

      it('should not exist', () => {
        component.vorgang = createVorgangWithEingangResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidenIconButton);
      });

      it('should exist with input', () => {
        const button: ButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(fixture, bescheidenIconButton);

        expect(button.stateResource).toBe(state);
      });

      it('should have "Vorgang bescheiden" tooltip', () => {
        tooltipExistsWithText(fixture, bescheidenIconButton, 'Vorgang bescheiden');
      });

      it('should have "Vorgang bescheiden fortsetzen" tooltip', () => {
        component.vorgang = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.BESCHEIDEN,
          VorgangWithEingangLinkRel.BESCHEID_DRAFT,
        ]);

        fixture.detectChanges();

        tooltipExistsWithText(fixture, bescheidenIconButton, 'Vorgang bescheiden fortsetzen');
      });

      it('should call bescheiden on click', () => {
        component.bescheiden = jest.fn();

        triggerEvent({ fixture, name: 'clickEmitter', elementSelector: bescheidenIconButton });

        expect(component.bescheiden).toHaveBeenCalled();
      });
    });
  });
});
